
package com.ibm.abbot.tester.swt;

import abbot.Log;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Combo.
 */
public class ComboTester extends CompositeTester {
	Object objT;
	boolean boolT;
	int intT0, intT1, intT2;

	/* Widget properties that are obtainable by member getter functions */
	protected int itemCount;
	protected int itemHeight;
	protected String[] items;
	protected Point selection;
	protected int selectionIndex;
	protected String text;

	public synchronized int getTextLimit(Combo combo) {
		getProperties(combo);
		return textLimit;
	}
	protected int textHeight;
	protected int textLimit;
	
	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Combo combo){
		super.getProperties(combo);
		Robot.syncExec(combo.getDisplay(),this,new Runnable(){
			public void run(){
				itemCount = combo.getItemCount();
				itemHeight = combo.getItemHeight();
				items = combo.getItems();
				selection = combo.getSelection();
				selectionIndex = combo.getSelectionIndex();
				text = combo.getText();
				textHeight = combo.getTextHeight();
				textLimit = combo.getTextLimit();					
			}
		});
	}
	

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */
	public synchronized int getItemCount(Combo combo) {
		getProperties(combo);
		return itemCount;
	}

	public synchronized int getItemHeight(Combo combo) {
		getProperties(combo);
		return itemHeight;
	}

	public synchronized String[] getItems(Combo combo) {
		getProperties(combo);
		return items;
	}

	public synchronized Point getSelection(Combo combo) {
		getProperties(combo);
		return selection;
	}

	public synchronized int getSelectionIndex(Combo combo) {
		getProperties(combo);
		return selectionIndex;
	}

	public synchronized String getText(Combo combo) {
		getProperties(combo);
		return text;
	}

	public synchronized int getTextHeight(Combo combo) {
		getProperties(combo);
		return textHeight;
	}

	/* End getters */
	
	/** 
	 * Drop down the menu for the given Combo box 
	 * WARNING: This method is platform-dependent.
	 */
	protected void dropDownCombo(Combo combo){
		int style = getStyle(combo);
		final int BUTTON_SIZE = 16;
		if((style&SWT.DROP_DOWN)==SWT.DROP_DOWN){
			Rectangle bounds = getGlobalBounds(combo);
			mouseMove(	bounds.x+bounds.width-BUTTON_SIZE/2,
						bounds.y+bounds.height-BUTTON_SIZE/2);
			mousePress(SWT.BUTTON1);
			mouseRelease(SWT.BUTTON1);
			actionWaitForIdle(combo.getDisplay());
		}
	}
	
	/** Move the mouse pointer over the item with the given index **/
	public /*protected*/ void mouseMoveIndex(Combo combo, int index){
		int style = getStyle(combo);
		if((style&SWT.DROP_DOWN)==SWT.DROP_DOWN){
			// TODO Add code to scroll down and move the mouse pointer;  
			// may not be possible b/c combo.getVerticalBar() returns null even when the 
			// bar on the drop-down is visible
		}	
		else{// SWT.SIMPLE
			// TODO Add code to scroll so item is visible and move
			// pointer over item
		}
	}
	
	/**
	 * Select the item from the Combo at the given index.
	 * 
	 * @param combo Combo from which to select
	 * @param index Index of item to select
	 */
	public void actionSelectIndex(final Combo combo, final int index){
		actionFocus(combo);	
		mouseMoveIndex(combo,index);
		Robot.syncExec(combo.getDisplay(),null,new Runnable(){
			public void run(){
				combo.select(index);	
			}
		});						
		actionWaitForIdle(combo.getDisplay());
	}	
	
	/**
	 * Select the given item from the Combo.
	 * 
	 * @param combo Combo from which to select
	 * @param item String to select
	 */
	public synchronized void actionSelectItem(final Combo combo, String item){
		Robot.syncExec(combo.getDisplay(),this,new Runnable(){
			public void run(){
				objT = combo.getItems();
			}
		});	
		String[] items = (String[])objT;
		int i=0;
		while(true){
			if(i==items.length){
				Log.debug("actionSelectItem: item \""+item+"\" not found");
				break;
			}
			if(item.equals(items[i])){
				actionSelectIndex(combo,i);
				break;						
			}
			i++;
		}
	}
	
	/**
	 * Returns the item at the given index.
	 * 
	 * @param combo Combo from which to obtain the item	
	 * @param index Index of the item 
	 * @return the item at the given index, or null if index is out-of-bounds
	 */
	public synchronized String getItem(final Combo combo, final int index){
		objT = null;
		Robot.syncExec(combo.getDisplay(),this,new Runnable(){
			public void run(){
				try{objT = combo.getItem(index);}
				catch(Exception ignored){}
			}
		});
		return (String)objT;
	}
	
	/**
	 * Indicates if the given index is the index of the currently selected item.
	 * 
	 * @param combo Combo to check
	 * @param index Index of the item to check
	 * @return whether the item at the given index is selected
	 */
	public synchronized boolean assertIndexSelected(final Combo combo, int index){
		Robot.syncExec(combo.getDisplay(),this,new Runnable(){
			public void run(){
				intT = combo.getSelectionIndex();
			}
		});
		return intT == index;
	}
	
	/**
	 * Indicates if the given item is currently selected.
	 * 
	 * @param combo Combo to check
	 * @param item Item to check
	 * @return whether the given item is selected
	 */
	public synchronized boolean assertItemSelected(final Combo combo, String item){
		objT = null;
		Robot.syncExec(combo.getDisplay(),this,new Runnable(){
			public void run(){
				int index = combo.getSelectionIndex();
				if(index!=-1)
					objT = combo.getItem(index);
			}
		});		
		return objT!=null && ((String)objT).equals(item);		
	}	 
}
