
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Composite.
 */
public class CompositeTester extends ControlTester {

	/* Widget properties that are obtainable by member getter functions */
	protected Control[] children;
	protected Layout layout;
	protected Control[] tabList;

	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Composite c){
		super.getProperties(c);
		Robot.syncExec(c.getDisplay(),this,new Runnable(){
			public void run(){
				children = c.getChildren();
				layout = c.getLayout();
				tabList = c.getTabList();
			}
		});
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */	
	public synchronized Control[] getChildren(Composite c) {
		getProperties(c);
		return children;
	}

	public synchronized Layout getLayout(Composite c) {
		getProperties(c);
		return layout;
	}

	public synchronized Control[] getTabList(Composite c) {
		getProperties(c);
		return tabList;
	}
	/* End getters */
}
