package com.ibm.abbot.tester.swt;


import java.util.*;
import org.eclipse.swt.widgets.*; 
import org.eclipse.swt.graphics.*;

// FOR TESTING----------------
//import searcher.gui.*;
//import searcher.gui.test.*;
//import org.eclipse.swt.*;
//import java.awt.Robot;
//import java.awt.AWTException;
// ---------------------------

/*
 * @author Kevin T Dale
 * 
 * Provides a means to find the coordinates of an SWT control in display-space, 
 * given an SWT Display object. 
 */

public class ControlFinder {
	Hashtable controlsParents;
	
	/**
	 * Creates a new instance of this class, given a Display object.
	 *  
	 * @param display the display object of the SWT application
	 */		
	 public ControlFinder(Display display){
		controlsParents = new Hashtable();
				
		// add all controls recursively
		Shell[] shells = display.getShells();
		for(int i=0; i<shells.length; i++)
			this.addControl(shells[i], display);						
	}
	
	/**
	 * Returns the top-left corner of a Control in display-space 
	 * coordinates.	
	 * @param c Control to be found
	 */
	protected Point getPoint(Control c){
		Composite parent = null;
		Point p = c.getLocation();
		try{
			parent = (Composite)(controlsParents.get(c));
		}
		catch(Exception ignored){//classCastException
		}
		if(parent==null)//parent is the display, not a composite
			return p;
		else
			return parent.toDisplay(p);
	}
	
	/**
	 * Returns the bounding box in display-space coordinates for a 
	 * given Control.
	 * @param c Control to be found
	 */
	protected Rectangle getRectangle(Control c){
		
		Composite parent = null;
		Rectangle r = c.getBounds();
		try{
			parent = (Composite)(controlsParents.get(c));
		}
		catch(Exception ignored){//classCastException
		}
		
		if(parent==null)
			return r;
		else{
			Point topLeftP = new Point(r.x,r.y);						// parent coords
			Point bottomRightP= new Point(r.x +r.width,r.y+r.height);  
			Point topLeftD = parent.toDisplay(topLeftP);				// display coords
			Point bottomRightD = parent.toDisplay(bottomRightP);				
			return new Rectangle(topLeftD.x,topLeftD.y,bottomRightD.x-topLeftD.x,bottomRightD.y-topLeftD.y);
		}
	}
	
	/**
	 * Adds a control, and all of its children controls, to the HashTable
	 * of control/parent pairs
	 *  
	 * @param c Control to add
	 * @param parent parent Object to add
	 */
	protected void addControl(Control c, Object parent){
		if(c==null)
			return;
		if(parent==null)
			return;
		
		Control[] children = new Control[0];
		try{
			children = ((Composite)c).getChildren();
		}
		catch(Exception ignored){		// a classCastException, but if its not a composite		
		}						 		// then it has no children anyway
		for(int i=0; i<children.length;i++)
			this.addControl(children[i],c);
		
		controlsParents.put(c,parent);
	}
	
	/**
	 * Contains some tests for ControlFinder
	 */
	/*
	public static void main(String[] args) {
		Display display = new Display();
		Shell shell = new Shell(display, SWT.TITLE | SWT.CLOSE);
		SearchPanel sp = new SearchPanel(shell);
		sp.setSearcher(new TestSearcher());
		shell.open();
	

		// run through some robot tests to make sure that
		// points and Rects are OK
		try{
			Robot r = new Robot();
			r.setAutoDelay(600);
			ControlFinder cf = new ControlFinder(display);
			Point p1,p2,p3;
			Rectangle rect;
			Control c;
			
			LinkedList controls = new LinkedList();
			controls.add(sp.findButton);
			controls.add(sp.queryArea);
			controls.add(sp.searchLabel);
			controls.add(sp.queryField);
			controls.add(sp.resultArea);
			controls.add(sp.resultTable);
			controls.add(shell);
			
			for(int i=0; i<controls.size();i++){
				c = (Control)controls.get(i);
				p1 = cf.getPoint(c);
				rect = cf.getRectangle(c);
				p2 = new Point(rect.x,rect.y);
				p3 = new Point(p1.x+rect.width,p1.y+rect.height);
				r.mouseMove(p1.x, p1.y);
				r.mouseMove(p2.x, p2.y);
				r.mouseMove(p3.x, p3.y);
			}
		}
		catch(AWTException awte){
			awte.printStackTrace();
		}
	
	
		while (!shell.isDisposed()) {
			if (! display.readAndDispatch()) {
				display.sleep();
			}
				
		}
	}
	*/
}
