
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type CoolBar.
 */
public class CoolBarTester extends CompositeTester{
	/* Widget properties that are obtainable by member getter functions */	
	protected CoolItem item;
	protected int itemCount;
	protected int[] itemOrder;
	protected CoolItem[] items;
	protected Point[] itemSizes;
	protected boolean locked;
	protected int[] wrapIndices;

	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final CoolBar bar){
		super.getProperties(bar);
		Robot.syncExec(bar.getDisplay(),this,new Runnable(){
			public void run(){
				itemCount = bar.getItemCount();
				itemOrder = bar.getItemOrder();
				items = bar.getItems();
				itemSizes = bar.getItemSizes();
				locked = bar.getLocked();
				wrapIndices = bar.getWrapIndices();
			}
		});
	}
	
	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */
	public synchronized CoolItem getItem(CoolBar bar) {
		getProperties(bar);
		return item;
	}

	public synchronized int getItemCount(CoolBar bar) {
		getProperties(bar);
		return itemCount;
	}

	public synchronized int[] getItemOrder(CoolBar bar) {
		getProperties(bar);
		return itemOrder;
	}

	public synchronized CoolItem[] getItems(CoolBar bar) {
		getProperties(bar);
		return items;
	}

	public synchronized Point[] getItemSizes(CoolBar bar) {
		getProperties(bar);
		return itemSizes;
	}

	public synchronized boolean getLocked(CoolBar bar) {
		getProperties(bar);
		return locked;
	}

	public synchronized int[] getWrapIndices(CoolBar bar) {
		getProperties(bar);
		return wrapIndices;
	}

	public synchronized CoolItem getItem(final CoolBar bar, final int index){
		Robot.syncExec(bar.getDisplay(),this,new Runnable(){
			public void run(){
				item = bar.getItem(index);
			}
		});
		return item;
	}
	/* End getters */
}
