
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Label.
 */
public class LabelTester extends WidgetTester{
	/* Widget properties that are obtainable by member getter functions */
	protected int alignment;
	protected Image image;
	protected String text;

	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Label l){
		super.getProperties(l);
		Robot.syncExec(l.getDisplay(),this,new Runnable(){
			public void run(){
				alignment = l.getAlignment();
				image = l.getImage();
				text = l.getText();			
			}
		});
		
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */	
	public synchronized String getText(Label l){
		getProperties(l);
		return text;		
	}
	
	public synchronized Image getImage(Label l){
		getProperties(l);
		return image;
	}
	
	public synchronized int getAlignment(Label l){
		getProperties(l);
		return alignment;
	}

	/* End getters */

	public boolean assertTextEquals(Label label, String text) {
		return assertTextEquals(label, text, false);
	}

	/**
	 * Fixes problem observed running CCT under TestCollector:
	 * getText(label) returning text minus trailing whitespace
	 */
	public boolean assertTextEquals(
		Label label, String textToMatch, boolean trim) {
		String gotText = getText(label);
		if (gotText == null) {
			return (textToMatch == null);
		} else if (trim) {
			textToMatch = textToMatch.trim();
			gotText = gotText.trim();
		}
		return gotText.equals(textToMatch);
	}
	
}
