
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Scale.
 */
public class ScaleTester extends ControlTester {
	/* Widget properties that are obtainable by member getter functions */
	protected int minimum, maximum, increment, selection, pageIncrement;

	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Scale scale){
		super.getProperties(scale);
		Robot.syncExec(scale.getDisplay(),this,new Runnable(){
			public void run(){
				minimum = scale.getMinimum();
				maximum = scale.getMaximum();
				increment = scale.getIncrement();
				selection = scale.getSelection();
				pageIncrement = scale.getPageIncrement();					
			}
		});
	}
	
	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */	
	public synchronized int getIncrement(Scale scale) {
		getProperties(scale);
		return increment;
	}

	public synchronized int getMaximum(Scale scale) {
		getProperties(scale);
		return maximum;
	}

	public synchronized int getMinimum(Scale scale) {
		getProperties(scale);
		return minimum;
	}

	public synchronized int getPageIncrement(Scale scale) {
		getProperties(scale);
		return pageIncrement;
	}

	public synchronized int getSelection(Scale scale) {
		getProperties(scale);
		return selection;
	}
	/* End getters */
}
