
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.widgets.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Slider.
 */
public class SliderTester extends ScrollableTester {
	/* Widget properties that are obtainable by member getter functions */
	protected boolean enabled;
	protected int increment, maximum, minimum, pageIncrement,selection, thumb;

	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Slider slider){
		super.getProperties(slider);
		Robot.syncExec(slider.getDisplay(),this,new Runnable(){
			public void run(){
				enabled = slider.getEnabled();
				increment = slider.getIncrement();
				maximum = slider.getMaximum();
				minimum = slider.getMinimum();
				pageIncrement = slider.getPageIncrement();
				selection = slider.getSelection();
				thumb = slider.getThumb();
			}
		});
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */
	public synchronized boolean getEnabled(Slider slider) {
		getProperties(slider);
		return enabled;
	}

	public synchronized int getIncrement(Slider slider) {
		getProperties(slider);
		return increment;
	}

	public synchronized int getMaximum(Slider slider) {
		getProperties(slider);
		return maximum;
	}

	public synchronized int getMinimum(Slider slider) {
		getProperties(slider);
		return minimum;
	}

	public synchronized int getPageIncrement(Slider slider) {
		getProperties(slider);
		return pageIncrement;
	}

	public synchronized int getSelection(Slider slider) {
		getProperties(slider);
		return selection;
	}

	public synchronized int getThumb(Slider slider) {
		getProperties(slider);
		return thumb;
	}
	/* End getters */
}
