
package com.ibm.abbot.tester.swt;

import java.util.ArrayList;

import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;

/** 
 * Provides widget-specific actions for testing Tables.  Note that 
 * actionShowTableColumn, actionSelectTableColumn, and actionResizeTableColumn 
 * are platform-dependent and moreover, may not work with versions of 
 * SWT != 2.1.1.
 */
public class TableTester extends CompositeTester{
	/* Widget properties that are obtainable by member getter functions */
	protected TableColumn column;
	protected int columnCount,itemCount,itemHeight,selectionCount,selectionIndex,
		topIndex, gridLineWidth, headerHeight;
	protected TableColumn[] columns;
	protected boolean headerVisible, linesVisible;
	protected TableItem item;		
	protected TableItem[] items,selection;
		
	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */
	protected synchronized void getProperties(Table table){
		super.getProperties(table);
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
							
			}
		});	
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */	
	public synchronized TableItem getItem(final Table table, final int index){	
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
				item = table.getItem(index);
			}
		});
		return item;
	}
	
	public synchronized TableItem getItem(final Table table, final Point point){	
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
				item = table.getItem(point);
			}
		});
		return item;
	}
	
	public synchronized TableColumn getColumn(Table table) {
		getProperties(table);
		return column;
	}

	public synchronized int getColumnCount(Table table) {
		getProperties(table);
		return columnCount;
	}

	public synchronized TableColumn[] getColumns(Table table) {
		getProperties(table);
		return columns;
	}

	public synchronized int getGridLineWidth(Table table) {
		getProperties(table);
		return gridLineWidth;
	}

	public synchronized int getHeaderHeight(Table table) {
		getProperties(table);
		return headerHeight;
	}

	public synchronized boolean getHeaderVisible(Table table) {
		getProperties(table);
		return headerVisible;
	}

	public synchronized int getItemCount(Table table) {
		getProperties(table);
		return itemCount;
	}

	public synchronized int getItemHeight(Table table) {
		getProperties(table);
		return itemHeight;
	}

	public synchronized TableItem[] getItems(Table table) {
		getProperties(table);
		return items;
	}

	public synchronized boolean getLinesVisible(Table table) {
		getProperties(table);
		return linesVisible;
	}

	public synchronized TableItem[] getSelection(Table table) {
		getProperties(table);
		return selection;
	}

	public synchronized int getSelectionCount(Table table) {
		getProperties(table);
		return selectionCount;
	}

	public synchronized int getSelectionIndex(Table table) {
		getProperties(table);
		return selectionIndex;
	}

	public synchronized int getTopIndex(Table table) {
		getProperties(table);
		return topIndex;
	}
	/* End getters */
	
	/** Returns all selected items in the given table **/
	public synchronized TableItem[] getSelectedTableItems(final Table table){
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
				objT = table.getSelection();				
			}
		});
		return (TableItem[])objT;		
	}
	
	/** Returns the indices of all selected items in the given table **/
	public synchronized int[] getSelectedTableIndices(final Table table){
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
				objT = table.getSelectionIndices();				
			}
		});
		return (int[])objT;		
	}
	
	/** Returns the all checked items in the given table **/
	public synchronized TableItem[] getCheckedTableItems(final Table table){
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
				ArrayList checked = new ArrayList();
				TableItem[] items = table.getItems();
				for(int i=0; i<items.length;i++){
					if(items[i].getChecked())
						checked.add(items[i]);			
				}
				objT = checked.toArray(new TableItem[checked.size()]);
			}
		});
		return (TableItem[])objT;		
	}

	/** Returns the indices of all checked items in the given table **/
	public synchronized int[] getCheckedTableIndices(final Table table){
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
				ArrayList checked = new ArrayList();
				TableItem[] items = table.getItems();
				for(int i=0; i<items.length;i++){
					if(items[i].getChecked())
						checked.add(new Integer(i));			
				}
				objT = checked.toArray(new Integer[checked.size()]);
			}
		});
		int[] res = new int[((Integer[])objT).length];
		for(int i=0; i<res.length;i++)
			res[i] = ((Integer[])objT)[i].intValue();
		return res;
	}
	
	/** Move the mouse pointer over the given TableItem **/
	protected synchronized void mouseMoveTableItem(final Table table, final TableItem item){
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
				table.setFocus();
				table.showItem(item);
				Rectangle bounds = WidgetLocator.getBounds(table,true);	
				int col0Width = table.getColumn(0).getWidth();
				int itemHeight = table.getItemHeight();
				Point p = new Point(col0Width/2,itemHeight/2);
				TableItem itemAtPoint=null;
				while(true){
					if(p.y>bounds.y+bounds.height)
						break;
					itemAtPoint = table.getItem(p);		
					if(itemAtPoint==item)
						break;		
					p.y+=itemHeight;
				}		
				if(itemAtPoint==item)
					mouseMove(bounds.x+p.x,bounds.y+p.y);
			}			
		});
		actionWaitForIdle(table.getDisplay());			
	}
	
	/** Move the mouse pointer over the TableItem at the given index **/
	protected synchronized void mouseMoveTableIndex(final Table table, final int index){
		//synchronized(this){
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
				public void run(){
					objT = table.getItem(index);
				}
			});
		//}
		if(objT!=null)
			mouseMoveTableItem(table,(TableItem)objT);
	}
	
	/** Move the mouse pointer to the top of the given column **/
	protected void mouseMoveTopOfColumn(final Table table, final TableColumn col){
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				TableColumn[] cols = table.getColumns();
				int width = 0;
				int i;
				for(i=0; i<cols.length;i++){
					width+=cols[i].getWidth();
					if(col==cols[i])
						break;									
				}
				if(col==cols[i]){
					int height = table.getHeaderHeight()/2;
					width-=col.getWidth()/2;
					Point loc = WidgetLocator.getLocation(table);
					mouseMove(loc.x+width,loc.y+height);
				}
			}
		});
		actionWaitForIdle(table.getDisplay());			
	}
	
	
	/** Click an item in the given table.  **/
	public void actionClickTableItem(final Table table, final TableItem item){
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				table.showItem(item);
				Rectangle bounds = WidgetLocator.getBounds(table,true);	
				int col0Width = table.getColumn(0).getWidth();
				int itemHeight = table.getItemHeight();
				Point p = new Point(col0Width/2,itemHeight/2);
				TableItem itemAtPoint=null;
				while(true){
					if(p.y>bounds.y+bounds.height)
						break;
					itemAtPoint = table.getItem(p);		
					if(itemAtPoint==item)
						break;				
				}		
				if(itemAtPoint==item)
					actionClick(table,p.x,p.y);
			}			
		});
		actionWaitForIdle(table.getDisplay());
	}
	
	/** Click the item at the given index **/
	public synchronized void actionClickTableIndex(final Table table, final int index){
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
				objT = table.getItem(index);
			}
		});
		if(objT!=null)
			actionClickTableItem(table,(TableItem)objT);
	}

	/** Click the given column's header (if headers are visible) **/
	public synchronized void actionClickTableColumnHeader(final Table table, TableColumn column){
		Robot.syncExec(table.getDisplay(),this,new Runnable(){
			public void run(){
				objT = new Boolean(table.getHeaderVisible());
			}
		});
		if(((Boolean)objT).booleanValue()){
			actionShowTableColumn(table,column); // also positions pointer above header
			mousePress(SWT.BUTTON1);
			mouseRelease(SWT.BUTTON1);
			actionWaitForIdle(table.getDisplay());
		}	
	}
	
	/** Select an item from the given table, or deselect it if it was already selected. **/
	public void actionSelectTableItem(final Table table, final TableItem item){
		mouseMoveTableItem(table,item);
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				TableItem[] items = table.getItems();
				int i;
				for(i=0; i<items.length;i++){
					if(items[i]==item)
						break;
				}
				int index = i;
				if(items[i]==item){
					table.showItem(item);
					TableItem[] selectedItems = getSelectedTableItems(table);
					boolean selected =false;
					for(i=0; i<selectedItems.length;i++){
						if(selectedItems[i]==item){
							selected = true;
							table.deselect(i);	
							break;
						}
					}
					if(!selected)
						table.select(index);	
				}				
			}
		});
		actionWaitForIdle(table.getDisplay());
	}
	
	/** Select the item at the given index from the table, or deselect it if it was already selected.  **/
	public void actionSelectTableIndex(final Table table, final int index){
		mouseMoveTableIndex(table,index);
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				TableItem item = null;
				try{item = table.getItem(index);}				
				catch(Exception ignored){}
				if(item!=null){
					table.showItem(item);
					TableItem[] selectedItems = getSelectedTableItems(table);
					boolean selected =false;
					for(int i=0; i<selectedItems.length;i++){
						if(selectedItems[i]==item){
							selected = true;
							table.deselect(i);	
							break;
						}
					}
					if(!selected)
						table.select(index);	
				}				
			}
		});
		actionWaitForIdle(table.getDisplay());
	}	
	
	/** Put the item as high as possible in the given table's viewing window **/
	public void actionShowTableItem(final Table table, final TableItem item) {
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				TableItem[] items = table.getItems();
				int i;
				for(i=0; i<items.length;i++){
					if(items[i]==item)
						break;
				}
				if(items[i]==item){
					table.setTopIndex(i);	
				}									
			}
		});
		actionWaitForIdle(table.getDisplay());		
	}
	
	/** Put the item at the given index as high as possible in the table's viewing window **/
	public void actionShowTableIndex(final Table table, final int index){
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				table.setTopIndex(index);					
			}
		});
		actionWaitForIdle(table.getDisplay());		
	}
	
	/** 
	 * Scroll so that the given table column is visible, and place the mouse over the
	 * column's heading, if visible.
	 **/
	public void actionShowTableColumn(final Table table, final TableColumn column){
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				ScrollBar scrollBar = table.getHorizontalBar();
				
				int width = 0;
				int totalWidth = 0;
				TableColumn[] cols = table.getColumns();
				for(int i=0; i<cols.length;i++){
					totalWidth+=cols[i].getWidth();
				}
				for(int i=0; i<cols.length&&cols[i]!=column;i++){
					width+=cols[i].getWidth();					
				}
				width+=column.getWidth()/2;
				
				ScrollBarTester tester = (ScrollBarTester)WidgetTester.getTester(ScrollBar.class);
				Rectangle scrollBounds = tester.getBounds(scrollBar);
				
				int max = scrollBar.getMaximum();
				int min = scrollBar.getMinimum();
				
				int tableWidth = table.getBounds().width;
				int setScroll =(int)( (double)((double)width /(double)totalWidth)*(max-min))-tableWidth/2;
//				System.out.println(	"setScroll="+setScroll+" min="
//									+scrollBar.getMinimum()+" max="+scrollBar.getMaximum()
//									+" thumb="+scrollBar.getThumb()
//									+" width="+width+" totalWidth="+totalWidth
//									+" tableWidth="+tableWidth+"scrollWidth="+scrollBounds.width);
								
				tester.actionScrollSetSelection(scrollBar,setScroll);

				// now move mouse to top of column
				int selection = scrollBar.getSelection();			
				int thumb = scrollBar.getThumb();
				double thumbEdge = (double)((double)selection/(double)(max-min));
				int leftEdgeOfTableShowing = (int)(thumbEdge*totalWidth);
	
				Point tableLoc = WidgetLocator.getLocation(table);				
			
				//System.out.println("Selection="+selection+" leftEdge...="+leftEdgeOfTableShowing);
				int offset = width - leftEdgeOfTableShowing;
				if(offset>=scrollBounds.width){
					System.out.println("offset to large("+offset+")");	
					offset = scrollBounds.width-2;
				}
				mouseMove(	tableLoc.x+(offset),
							tableLoc.y+table.getHeaderHeight()/2);
			}
		});
		actionWaitForIdle(table.getDisplay());		
	}
	
	/** Check the check box for an item in the given table,, or uncheck it if it 
	 *  was already checked.  
	 **/
	public void actionCheckTableItem(final Table table, final TableItem item){
		mouseMoveTableItem(table,item);
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				table.showItem(item);
				if(item.getChecked())
					item.setChecked(false);
				else
					item.setChecked(true);					
			}
		});
		actionWaitForIdle(table.getDisplay());		
	}
	
	/** Check the check box for the item at the given index in the table, or
	 *  uncheck it if it was already checked. 
	 **/
	public void actionCheckTableIndex(final Table table, final int index){
		mouseMoveTableIndex(table,index);
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				TableItem[] items = table.getItems();
				if(index>=0 && index<items.length){
					table.showItem(items[index]);
					if(items[index].getChecked())
						items[index].setChecked(false);
					else
						items[index].setChecked(true);					
				}						
			}
		});
		actionWaitForIdle(table.getDisplay());			
	}

	/** Resize the column in the given table **/
	public void actionResizeTableColumn(final Table table, final TableColumn column, final int width){
		actionShowTableColumn(table,column);
		mouseMoveTopOfColumn(table,column);
		Robot.syncExec(table.getDisplay(),null,new Runnable(){
			public void run(){
				table.setFocus();
				column.setWidth(width);
				// TODO need to make the column visible here and maybe move the mouse over it									
			}
		});
		actionWaitForIdle(table.getDisplay());		
		mouseMoveTopOfColumn(table,column);
		actionWaitForIdle(table.getDisplay()); 
	}

}
