
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;

/**
 * Provides widget-specific actions, assertions, and getter methods for
 * widgets of type Text.
 */
public class TextTester extends ScrollableTester{
	/* Widget properties that are obtainable by member getter functions */
	protected int caretLineNumber, caretPosition, charCount, lineCount,
		lineHeight, selectionCount, tabs, textLimit, topIndex, topPixel;
	protected Point caretLocation, selection;
	protected boolean doubleClickEnabled, editable;
	protected char echoChar;
	protected String lineDelimiter, selectionText, text;

	/**
	 * Sets the above properties to their current values for the given widget. 
	 * NOTE: This should be called in a block of code synchronized on this
	 * tester.
	 */	
	protected synchronized void getProperties(final Text t){
		super.getProperties(t);
		Robot.syncExec(t.getDisplay(),this,new Runnable(){
			public void run(){
				text = t.getText();
				caretLineNumber = t.getCaretLineNumber();
				caretPosition = t.getCaretPosition();
				charCount = t.getCharCount();
				lineCount = t.getLineCount();
				lineHeight = t.getLineHeight();
				selectionCount = t.getSelectionCount();
				tabs = t.getTabs();
				textLimit = t.getTextLimit();
				topIndex = t.getTopIndex();
				topPixel = t.getTopPixel();
				//caretLocation = t.getCaretLocation();
				selection = t.getSelection();
				doubleClickEnabled = t.getDoubleClickEnabled();
				editable = t.getEditable();
				echoChar = t.getEchoChar();
				lineDelimiter = t.getLineDelimiter();
				selectionText = t.getSelectionText();
			}
		});			
	}

	/**
	 * These getter methods return a particular property of the given widget.
	 * @see the corresponding member function in class Widget   
	 */ 
	/* Begin getters */	
	public synchronized String getText(	final Text t, final int start, 
										final int end){
		Robot.syncExec(t.getDisplay(),this,new Runnable(){
			public void run(){
				text = t.getText(start,end);
			}
		});
		return text;										
	}

	public synchronized int getCaretLineNumber(Text t) {
		getProperties(t);
		return caretLineNumber;
	}

	/** 
	 * Currently returns null.  There is a bug with SWT 3.0
	 * that causes all sorts of trouble when text.getCaretLocation()
	 * is called.
	 */
	public synchronized Point getCaretLocation(Text t) {
		getProperties(t);
		return caretLocation;
	}

	public synchronized int getCaretPosition(Text t) {
		getProperties(t);
		return caretPosition;
	}
	
	public synchronized int getCharCount(Text t) {
		getProperties(t);
		return charCount;
	}

	public synchronized boolean getDoubleClickEnabled(Text t) {
		getProperties(t);
		return doubleClickEnabled;
	}

	public synchronized char getEchoChar(Text t) {
		getProperties(t);
		return echoChar;
	}

	public synchronized boolean getEditable(Text t) {
		getProperties(t);
		return editable;
	}

	public synchronized int getLineCount(Text t) {
		getProperties(t);
		return lineCount;
	}

	public synchronized String getLineDelimiter(Text t) {
		getProperties(t);
		return lineDelimiter;
	}

	public synchronized int getLineHeight(Text t) {
		getProperties(t);
		return lineHeight;
	}

	public synchronized Point getSelection(Text t) {
		getProperties(t);
		return selection;
	}

	public synchronized int getSelectionCount(Text t) {
		getProperties(t);
		return selectionCount;
	}

	public synchronized String getSelectionText(Text t) {
		getProperties(t);
		return selectionText;
	}

	public synchronized int getTabs(Text t) {
		getProperties(t);
		return tabs;
	}

	public synchronized String getText(Text t) {
		getProperties(t);
		return text;
	}

	public synchronized int getTextLimit(Text t) {
		getProperties(t);
		return textLimit;
	}

	public synchronized int getTopIndex(Text t) {
		getProperties(t);
		return topIndex;
	}

	public synchronized int getTopPixel(Text t) {
		getProperties(t);
		return topPixel;
	}
	/* End getters */

	public void actionEnterText(final Text widget, final String text){
		actionFocus(widget);
		Robot.syncExec(widget.getDisplay(),null,new Runnable(){
			public void run(){
				widget.setText(text);
			}
		});
		actionWaitForIdle(widget.getDisplay());
	}

	public void actionSelect(final Text widget, final int start, final int end){
		actionFocus(widget);
		Robot.syncExec(widget.getDisplay(),null,new Runnable(){
			public void run(){
				widget.setSelection(start,end);		
			}
		});
		actionWaitForIdle(widget.getDisplay());
	}
//	
//	public synchronized String getText(final Text widget){
//		objT = null;
//		widget.getDisplay().syncExec(new Runnable(){
//			public void run(){
//				objT = widget.getText();
//			}
//		});
//		
//		return (String)objT;
//	}
//	
//	public String getText(Text widget, int start, int end){
//		String text = getText(widget);
//		if(text==null)
//			return null;
//		return text.substring(start,end);
//	}
//	
	public boolean assertTextEquals(Text widget, String text){
		String gotText = getText(widget);
		if (gotText==null)
			return text==null;
		return gotText.equals(text);
	}
	
	//	Did NOT implement click(Text,int)...
	// MAYBE HAVE AN ASSERTION FOR A RANGE OF TEXT???
	
	
	
}
