package com.ibm.abbot.examples;

import junit.framework.*;

import com.ibm.abbot.tester.swt.*;
import com.ibm.abbot.script.*;
import com.ibm.abbot.*;
//import com.ibm.junit.extensions.AssertUtils;

import org.eclipse.swt.widgets.*;

/** 
 * This is a demonstration of Abbot's support for hand-coded JUnit tests of SWT
 * applications.  See <url>http://www.sys-con.com/java/articleprint.cfm?id=1940</url>
 * for more info.  
 */
public class CelsiusConverterTest extends TestCase {
	static WidgetTester tester;
	static Thread appThread;
	static boolean startedApp = false;
	static final int DELAY = 300;
		
	public CelsiusConverterTest(String name) { 
			super(name);
	} 	
	
	public static Test suite() {
			return new TestSuite(CelsiusConverterTest.class, CelsiusConverterTest.class.getName());
	}

	public void setUp(){
		// start GUI in another thread 
		if(!startedApp){
			appThread = new Thread(){
				public void run(){
				   CelsiusConverter.main(null);
			   }
			};
			appThread.setName("AppThread");
			appThread.start();	
		
			startedApp = true;
		}		
						
		// make sure GUI is showing
		tester = WidgetTester.getTester(Shell.class);
		WidgetTester.waitForFrameShowing("Convert Celsius to Fahrenheit");
		WidgetReference ref = new WidgetReference(null,Shell.class,
									null,null,null,"Convert Celsius to Fahrenheit");
		Shell shell = null;
		try{
		shell = (Shell)DefaultWidgetFinder.getFinder().findWidget(ref);
		}catch(MultipleWidgetsFoundException mwfe){
			mwfe.printStackTrace();
		}catch(WidgetNotFoundException wnfe){
			wnfe.printStackTrace();
		}
		tester.actionFocus(shell);
	}

	public void testNegativeNumberInput(){
		// get references to widgets based on their name and text fields
		Text tempCelsius = null;
		Button convertButton =null;
		Label fahrenheitLabel = null;
		try{
			WidgetReference ref = new WidgetReference("tempCelsius", Text.class,"tempCelsius");
			tempCelsius = (Text)DefaultWidgetFinder.getFinder().findWidget(ref); 
			ref = new WidgetReference("Convert", Button.class,"Convert");
			convertButton = (Button)DefaultWidgetFinder.getFinder().findWidget(ref);
			ref = new WidgetReference("fahrenheitLabel", Label.class,"fahrenheitLabel");
			fahrenheitLabel = (Label)DefaultWidgetFinder.getFinder().findWidget(ref);
		}catch(MultipleWidgetsFoundException mwfe){
			mwfe.printStackTrace();
		}catch(WidgetNotFoundException wnfe){
			wnfe.printStackTrace();
		}		

		// execute the necessary actions
		tester =WidgetTester.getTester(tempCelsius);
		((TextTester)tester).actionEnterText(tempCelsius,"-45");
		tester.actionClick(convertButton);
		WidgetTester.delay(DELAY);	
						
		// check the state of the GUI after actions are performed				
		tester = WidgetTester.getTester(fahrenheitLabel);
		String wanted = "Fahrenheit: \t-49\u00B0";
		String got = ((LabelTester)tester).getText(fahrenheitLabel);
		// test trimmed strings
		//AssertUtils.assertEquals("Temperature not converted correctly", wanted, got, true);
		assertEquals("Temperature not converted correctly", wanted.trim(),got.trim());
	}
	
	public void testBadInput(){
		Label fahrenheitLabel = null, celsiusLabel = null;
		Text tempCelsius = null;
		Button convertButton = null;
		try{
			WidgetReference ref = new WidgetReference("fahrenheitLabel",Label.class,"fahrenheitLabel");
			fahrenheitLabel = (Label)DefaultWidgetFinder.getFinder().findWidget(ref);
			ref = new WidgetReference("celsiusLabel",Label.class,"celsiusLabel");
			celsiusLabel = (Label)DefaultWidgetFinder.getFinder().findWidget(ref);
			ref = new WidgetReference("tempCelsius", Text.class,"tempCelsius");
			tempCelsius = (Text)DefaultWidgetFinder.getFinder().findWidget(ref); 
			ref = new WidgetReference("Convert", Button.class,"Convert");
			convertButton = (Button)DefaultWidgetFinder.getFinder().findWidget(ref);
		}catch(MultipleWidgetsFoundException mwfe){
			mwfe.printStackTrace();
		}catch(WidgetNotFoundException wnfe){
			wnfe.printStackTrace();
		}
		
		tester = WidgetTester.getTester(Label.class);
		String fTemp = ((LabelTester)tester).getText(fahrenheitLabel);	
		String cTemp = ((LabelTester)tester).getText(celsiusLabel);
		
		// nothing should change if the input to the text is not parse-able as 
		// a double
		tester = WidgetTester.getTester(Text.class);
		((TextTester)tester).actionEnterText(tempCelsius," HELLO ");
		tester.actionClick(convertButton);
		WidgetTester.delay(DELAY);
		
		tester = WidgetTester.getTester(Label.class);

		// test trimmed strings
		assertTrue("Fahrenheit output changed for bad input",
					((LabelTester)tester).assertTextEquals(fahrenheitLabel,fTemp, true));
		assertTrue("Celsius output changed for bad input",
					((LabelTester)tester).assertTextEquals(celsiusLabel,cTemp, true));

	}		

	public void testChangePrecision(){
		// the output should update to reflect a higher precision after making
		// a change, even with no new input
		MenuItem item2 = null;
		Text tempCelsius = null;
		Button convertButton = null;
		Label fahrenheitLabel = null;		
		try{
			WidgetReference ref = new WidgetReference("precision3",MenuItem.class,"2");
			item2 = (MenuItem)DefaultWidgetFinder.getFinder().findWidget(ref);
			ref = new WidgetReference("tempCelsius", Text.class,"tempCelsius");
			tempCelsius = (Text)DefaultWidgetFinder.getFinder().findWidget(ref); 
			ref = new WidgetReference("Convert", Button.class,"Convert");
			convertButton = (Button)DefaultWidgetFinder.getFinder().findWidget(ref);		
			ref = new WidgetReference("fahrenheitLabel",Label.class,"fahrenheitLabel");
			fahrenheitLabel = (Label)DefaultWidgetFinder.getFinder().findWidget(ref);
		}catch(MultipleWidgetsFoundException mwfe){
			mwfe.printStackTrace();
		}catch(WidgetNotFoundException wnfe){
			wnfe.printStackTrace();
		}
		
		// precision is at 0
		tester = WidgetTester.getTester(Text.class);
		((TextTester)tester).actionEnterText(tempCelsius,"25.23");
		tester.actionClick(convertButton);
		
		// now update precision and make sure output fields update too
		tester.actionSelectMenuItem(item2);
		WidgetTester.delay(DELAY);	
		
		tester = WidgetTester.getTester(Label.class);
		assertTrue("Failed to reflect change in precision",
					((LabelTester)tester).assertTextEquals(
					fahrenheitLabel,"Fahrenheit: \t77.41\u00B0"));
	
	}

	public void testHighPrecision(){
		MenuItem item3 = null;
		Text tempCelsius = null;
		Button convertButton = null;
		Label fahrenheitLabel = null;
		try{
			WidgetReference ref = new WidgetReference("precision3",MenuItem.class,"3");
			item3 = (MenuItem)DefaultWidgetFinder.getFinder().findWidget(ref);
			ref = new WidgetReference("tempCelsius", Text.class,"tempCelsius");
			tempCelsius = (Text)DefaultWidgetFinder.getFinder().findWidget(ref); 
			ref = new WidgetReference("Convert", Button.class,"Convert");
			convertButton = (Button)DefaultWidgetFinder.getFinder().findWidget(ref);		
			ref = new WidgetReference("fahrenheitLabel",Label.class,"fahrenheitLabel");
			fahrenheitLabel = (Label)DefaultWidgetFinder.getFinder().findWidget(ref);
		}catch(MultipleWidgetsFoundException mwfe){
			mwfe.printStackTrace();
		}catch(WidgetNotFoundException wnfe){
			wnfe.printStackTrace();
		}		
		tester.actionSelectMenuItem(item3);
		tester = WidgetTester.getTester(tempCelsius);
		((TextTester)tester).actionEnterText(tempCelsius,"-45.543");
		tester.actionClick(convertButton);
		WidgetTester.delay(DELAY);	
				
		tester = WidgetTester.getTester(Label.class);
		assertTrue("Failed to show answer with proper precision",
					((LabelTester)tester).assertTextEquals(
					fahrenheitLabel,"Fahrenheit: \t-49.977\u00B0"));
	
	}
}
