
package com.ibm.abbot.tester.swt;

import org.eclipse.swt.graphics.*;
import org.eclipse.swt.widgets.*;

import org.eclipse.swt.events.*;
import org.eclipse.swt.*;

import junit.extensions.abbot.*;


/** JUnit test fixture for testing the functionality of org.ibm.abbot.tester.swt.Robot */
public class RobotTest extends WidgetTestFixture {
	boolean clicked;
	boolean focused;
	boolean flag;
	
	public RobotTest(String name){
		super(name);
	}

	/* TODO Test drag-and-drop operations */


	// helper classes for monitoring key/mouse events
	private class KeyWatcher extends KeyAdapter {
	   public boolean gotPress = false;
	   public boolean gotRelease = false;
	   public boolean gotTyped = false;
	   public int keyCode = 0;
	   public char character;
	   public int modifiers = 0;
	   public void keyPressed(KeyEvent ke) {
		   gotPress = true;
		   keyCode = ke.keyCode;
		   modifiers = ke.stateMask;
		   character = ke.character;
	   }
	   public void keyReleased(KeyEvent ke) {
		   gotRelease = true;
		   if(gotPress)
			 gotTyped = true;
		   character = ke.character;
	   }
	   public void keyTyped(KeyEvent ke) {
		   gotTyped = true;
		   character = ke.character;
	   }
	};
	
	private class MouseWatcher extends MouseAdapter{
		public boolean gotPress = false;
		public boolean gotRelease = false;
		public boolean gotClick = false;
		public boolean gotDoubleClick = false;
		public boolean popupTrigger = false;
		public int clickCount = 0;
		public Widget source = null;
		public int modifiers = 0;
		public Point where = null;
	
		public void mouseDown(MouseEvent me){
			where = new Point(me.x,me.y);
			source = me.widget;
			gotPress = true;
			modifiers = me.stateMask;			
		}
		public void mouseUp(MouseEvent me){
			where = new Point(me.x,me.y);
			source = me.widget;
			modifiers = me.stateMask;
			gotRelease = true;
			if(gotPress)
				mouseClick(me);
		}
		public void mouseClick(MouseEvent me){
			where = new Point(me.x,me.y);
			if(Robot.POPUP_MASK==SWT.BUTTON2 && me.button==2 ||
			   Robot.POPUP_MASK==SWT.BUTTON3 && me.button==3)
				popupTrigger = true;
			gotClick = true;
			clickCount++;
			modifiers = me.stateMask;			
		}
		public void mouseDoubleClick(MouseEvent me){
			source = me.widget;
			modifiers = me.stateMask;			
			where = new Point(me.x,me.y);
			gotDoubleClick = true;
			if(Robot.POPUP_MASK==SWT.BUTTON2 && me.button==2 ||
			   Robot.POPUP_MASK==SWT.BUTTON3 && me.button==3)
				popupTrigger = true;
		}
	};

	// NOTE: uses hard-coded paths for images that are used
	public static final String TEST_IMAGE_DIR = "d:\\eclipse\\devspaces\\W5.0\\abbotforswt\\test\\abbot\\swt";
	public void testImageCapture(){
		//Shell shell = new Shell(display, SWT.SHELL_TRIM);
		Shell shell = createShell();
		shell.setBackground(new Color(shell.getDisplay(),255,255,255));
		final int X0 = 100;
		final int Y0 = 100;
		Image gif = new Image(shell.getDisplay(),TEST_IMAGE_DIR+"\\image.gif");
		Image jpg = new Image(shell.getDisplay(),TEST_IMAGE_DIR+"\\image.jpg");
		
		//shell.setLayout(new RowLayout(SWT.HORIZONTAL));
		Label label0 = new Label(shell,SWT.CENTER);
		label0.setImage(gif);
		Label label1 = new Label(shell,SWT.CENTER);
		label1.setImage(jpg);
		//showShell(shell);
		showShell(shell);
		shell.setSize(477,189);
		Robot.waitForIdle(shell.getDisplay());
			
		Image copy0 = robot.capture(label0,false);
		Image copy1 = robot.capture(label1,false);
			
		SWTImageComparator ic = new SWTImageComparator();
		assertTrue("Captured wrong image (0)",
				   ic.compare(robot.capture(label0), gif) == 0);		
		assertTrue("Captured wrong image (1)",
				   ic.compare(robot.capture(label1), jpg) == 0);	
		
		hideShell(shell);
	}

	public void testMenuSelection(){
		Shell shell = createShell();
		Menu bar = new Menu(shell,SWT.BAR);
		shell.setMenuBar(bar);
		MenuItem file = new MenuItem(bar,SWT.CASCADE);
		file.setText("File");
		Menu menu = new Menu(shell,SWT.DROP_DOWN);
		file.setMenu(menu);
		MenuItem one = new MenuItem(menu,SWT.CHECK);
		one.setText("One");
		MenuItem two = new MenuItem(menu,SWT.CHECK);
		two.setText("Two");
		two.setData("name","Two");
		MenuItem  subMenuItem = new MenuItem(menu,SWT.CASCADE);
		subMenuItem.setText("Sub-Menu");
		Menu subMenu = new Menu(shell,SWT.DROP_DOWN);
		subMenuItem.setMenu(subMenu);	
		MenuItem sub1 = new MenuItem(subMenu,SWT.CHECK);
		sub1.setText("Sub1");
		final MenuItem sub2 = new MenuItem(subMenu,SWT.CHECK);
		sub2.setText("Sub2");
		sub2.setData("name","Sub2");
		
		showShell(shell);
		
		two.setSelection(false);
		sub2.setSelection(false);
		robot.selectMenuItem(two);
		Robot.waitForIdle(shell.getDisplay());
		assertTrue("MenuItem Two not selected",two.getSelection());		
		robot.selectMenuItem(sub2);
		Robot.waitForIdle(shell.getDisplay());
		assertTrue("MenuItem Sub2 not selected",sub2.getSelection());
		
		two.setSelection(false);
		sub2.setSelection(false);
		robot.selectMenuItemByText(bar, "Two");
		Robot.waitForIdle(shell.getDisplay());
		assertTrue("MenuItem not selected by text",two.getSelection());
		robot.selectMenuItemByText(bar, "Sub2");
		Robot.waitForIdle(shell.getDisplay());
		assertTrue("MenuItem not selected by text",two.getSelection());

		hideShell(shell);
				
	}

	public void testPopupMenuSelection(){
		Shell shell = createShell();
		Menu popup = new Menu(shell,SWT.POP_UP);
		shell.setMenu(popup);
		MenuItem file = new MenuItem(popup,SWT.CASCADE);
		file.setText("File");
		Menu menu = new Menu(shell,SWT.DROP_DOWN);
		file.setMenu(menu);
		MenuItem one = new MenuItem(menu,SWT.CHECK);
		one.setText("One");
		MenuItem two = new MenuItem(menu,SWT.CHECK);
		two.setText("Two");
		two.setData("name","Two");
		MenuItem  subMenuItem = new MenuItem(menu,SWT.CASCADE);
		subMenuItem.setText("Sub-Menu");
		Menu subMenu = new Menu(shell,SWT.DROP_DOWN);
		subMenuItem.setMenu(subMenu);	
		MenuItem sub1 = new MenuItem(subMenu,SWT.CHECK);
		sub1.setText("Sub1");
		final MenuItem sub2 = new MenuItem(subMenu,SWT.CHECK);
		sub2.setText("Sub2");
		sub2.setData("name","Sub2");

		showShell(shell);

		two.setSelection(false);
		sub2.setSelection(false);
		robot.selectMenuItem(two);
		Robot.waitForIdle(shell.getDisplay());
		assertTrue("MenuItem Two not selected",two.getSelection());	
		robot.selectMenuItem(sub2);
		Robot.waitForIdle(shell.getDisplay());
		assertTrue("MenuItem Sub2 not selected",sub2.getSelection());

		two.setSelection(false);
		sub2.setSelection(false);
		robot.selectMenuItemByText(popup, "Two");
		Robot.waitForIdle(shell.getDisplay());
		assertTrue("MenuItem not selected by text",two.getSelection());
		robot.selectMenuItemByText(popup, "Sub2");
		Robot.waitForIdle(shell.getDisplay());
		assertTrue("MenuItem not selected by text",two.getSelection());

		hideShell(shell);		
	}

	public void testClick(){
		Shell shell = createShell();
		MouseWatcher mw = new MouseWatcher();
		shell.addMouseListener(mw);
		
		showShell(shell);
		
		robot.click(shell);
		Robot.waitForIdle(shell.getDisplay());
		
		assertTrue("Never received press", mw.gotPress);
		assertEquals("Wrong event source", shell, mw.source);
		assertTrue("Never received release", mw.gotRelease);
		assertTrue("Never received click", mw.gotClick);
		assertEquals("No modifiers expected ",SWT.BUTTON1,mw.modifiers);					 
		assertTrue("Unexpected popup trigger", !mw.popupTrigger);		

		hideShell(shell);	
	}
	
	// tests double click 
	public void testMultipleClicks(){
		Shell shell = createShell();
		MouseWatcher mw = new MouseWatcher();
		shell.addMouseListener(mw);
		
		showShell(shell);
				
		Rectangle bounds = shell.getBounds();				
		robot.click(shell,bounds.width/2,bounds.height/2,SWT.BUTTON1,2);
		Robot.waitForIdle(shell.getDisplay());
		
		assertTrue("Never received a double click",mw.gotDoubleClick);
		
		hideShell(shell);
	}
	
	public void testClickAt() {
		Shell shell = createShell();
		MouseWatcher mw = new MouseWatcher();
		Decorations dec = new Decorations(shell,SWT.NONE);
		dec.setSize(50,50);
		//label.setSize(50,50);
		dec.setBackground(new Color(shell.getDisplay(),255,255,255));
		dec.addMouseListener(mw);
				
		showShell(shell);
										
		int border = dec.getBorderWidth();								
		Point at = new Point(border,border);
						
		robot.click(dec,0,0);
		Robot.waitForIdle(shell.getDisplay());
		
		assertTrue("Unexpected popup trigger", !mw.popupTrigger);
		assertTrue("Never received press", mw.gotPress);
		assertEquals("Wrong event source", dec, mw.source);
		assertTrue("Never received release", mw.gotRelease);
		assertTrue("Never received click", mw.gotClick);
 		assertEquals("Wrong mouse coordinates", at, mw.where);
		assertEquals("No modifiers expected ",SWT.BUTTON1,mw.modifiers);					 
		assertTrue("Unexpected popup trigger", !mw.popupTrigger);				
		
		hideShell(shell);	
	}

	public void testClickPopup(){
		Shell shell = createShell();
		MouseWatcher mw = new MouseWatcher();
		shell.addMouseListener(mw);
		
		showShell(shell);
		
		int mask = Robot.POPUP_MASK;
		robot.click(shell,mask);
		Robot.waitForIdle(shell.getDisplay());
				
		assertTrue("Never received press", mw.gotPress);
		assertEquals("Wrong event source", shell, mw.source);
		assertTrue("Never received release", mw.gotRelease);
		assertTrue("Never received click", mw.gotClick);
		assertEquals("Wrong modifiers", mask, mw.modifiers);
		assertTrue("Should be popup trigger", mw.popupTrigger);		
		
		hideShell(shell);
	}

	public void testClickTertiary() {
		Shell shell = createShell();
		MouseWatcher mw = new MouseWatcher();
		shell.addMouseListener(mw);
		showShell(shell);

		int mask = Robot.TERTIARY_MASK;
		robot.click(shell,mask);
		Robot.waitForIdle(shell.getDisplay());
		
		assertTrue("Never received press", mw.gotPress);
		assertEquals("Wrong event source", shell, mw.source);
		assertTrue("Never received release", mw.gotRelease);
		assertTrue("Never received click", mw.gotClick);
		assertEquals("Wrong modifiers", mask, mw.modifiers);
		assertTrue("Unexpected popup trigger", !mw.popupTrigger);		

		hideShell(shell);
	}

	public void testFocusSingleShell(){
		Shell shell = createShell();
		Text tf1 = new Text(shell,SWT.BORDER);
		Text tf2 = new Text(shell,SWT.BORDER);
		tf1.setSize(150,20);
		tf2.setSize(150,20);
		tf1.setText("Default focus goes here");
		tf2.setText("Requested focus goes here");
		tf2.setLocation(0,30);
		
		showShell(shell);
			
		robot.focus(tf2);
		Robot.waitForIdle(shell.getDisplay());
	   	assertTrue("Default TextField should not have focus", !tf1.isFocusControl());
	   	assertTrue("Next TextField should have focus", tf2.isFocusControl());
	   
	   	robot.focus(tf1);
		Robot.waitForIdle(shell.getDisplay());
	   	assertTrue("First TextField should have focus", tf1.isFocusControl());
	
		hideShell(shell);
	}

	public void testFocusMultipleShells(){
		Shell shell1 = createShell();
		Shell shell2 = createShell();
		Text tf1 = new Text(shell1,SWT.BORDER);
		Text tf2 = new Text(shell1,SWT.BORDER);
		Text tf3 = new Text(shell2,SWT.BORDER);
		tf1.setSize(150,20);
		tf2.setSize(150,20);
		tf3.setSize(150,20);
		tf1.setText("Default focus goes here");
		tf2.setText("Requested focus goes here");
		tf3.setText("Next it goes here");
		tf2.setLocation(0,30);		

		
		showShell(shell1);
		showShell(shell2);
				
		robot.focus(tf2);
		Robot.waitForIdle(shell1.getDisplay());
		assertTrue("Default TextField should not have focus", !tf1.isFocusControl());
		assertTrue("Next TextField should have focus", tf2.isFocusControl());
   
		robot.focus(tf1);
		Robot.waitForIdle(shell1.getDisplay());
		assertTrue("First TextField should have focus", tf1.isFocusControl());
	
		robot.focus(tf3);
		Robot.waitForIdle(shell1.getDisplay());
		assertTrue("TextField on second Shell never got focus", tf3.isFocusControl());
	
		hideShell(shell1);
		hideShell(shell2);
	}


	public void testPlainKey(){
		Shell shell = createShell();
		Text tf1 = new Text(shell,SWT.BORDER);
		tf1.setSize(150,20);
		KeyWatcher kw = new KeyWatcher();
		tf1.addKeyListener(kw);
		
		showShell(shell);
		robot.focus(tf1);
		
		char c = 'a';
		robot.key(c);

		Robot.waitForIdle(shell.getDisplay());
	   assertTrue("Never received key press", kw.gotPress);
	   assertTrue("Never received release", kw.gotRelease);
	   assertTrue("Never received type", kw.gotTyped);
	   assertEquals("Wrong key code", c,kw.character);
	   assertEquals("Wrong modifiers", 0, kw.modifiers);
	
		hideShell(shell);
	}
	 
	// NOTE: part of the original test has been omitted, b/c we cannot use 
	// robot.keyString(String) to type unicode chars that do NOT have key mappings
	public void testKeyString(){
		Shell shell = createShell();
		Text tf1 = new Text(shell,SWT.BORDER);
		Text tf2 = new Text(shell,SWT.BORDER);
		tf1.setSize(150,20);
		tf2.setSize(150,20);
		tf2.setLocation(0,30);
		String keymap = "The quick brown fox jumped over the lazy dog";

		showShell(shell);
		
		robot.focus(tf1);
		robot.keyString(keymap);
		Robot.waitForIdle(shell.getDisplay());
		assertEquals("Wrong text typed", keymap, tf1.getText());
						
		hideShell(shell);
	
	}

	public void testKeyNonLetterChars(){
		Shell shell = createShell();
		Text tf1 = new Text(shell,SWT.BORDER);
		Text tf2 = new Text(shell,SWT.BORDER);
		tf1.setSize(150,20);
		tf2.setSize(150,20);
		tf2.setLocation(0,30);
		String keymap = "`~!@#$%^&*()-_=+[{]}\\|;:'\",<.>/?";

		showShell(shell);
		
		robot.focus(tf1);
		robot.keyString(keymap);
		Robot.waitForIdle(shell.getDisplay());
		for(int i=0; i<keymap.length();i++){
			assertEquals("Wrong text typed", keymap.charAt(i),tf1.getText().charAt(i));
		}

							
		hideShell(shell);
			
	}
	
	public void testKeyAccelerator(){
		Shell shell = createShell();
		Text tf1 = new Text(shell,SWT.BORDER);
		tf1.setSize(150,20);
		KeyWatcher kw = new KeyWatcher();
		tf1.addKeyListener(kw);
		
		showShell(shell);
		robot.focus(tf1);
		
		char c = 'a';
		int modifier = SWT.ALT;
		robot.key(c|SWT.ALT);

		Robot.waitForIdle(shell.getDisplay());
	   	assertTrue("Never received key press", kw.gotPress);
	   	assertTrue("Never received release", kw.gotRelease);
	   	assertTrue("Never received type", kw.gotTyped);
	   	assertEquals("Wrong key code", c,kw.character);
	   	assertEquals("Wrong modifiers", SWT.ALT, kw.modifiers);
	
		hideShell(shell);
	}

	public void testFindFocusOwner() {
		Shell shell = createShell();
		Text tf1 = new Text(shell,SWT.BORDER);
		Text tf2 = new Text(shell,SWT.BORDER);
		tf1.setSize(150,20);
		tf2.setSize(150,20);
		tf2.setLocation(0,30);

		showShell(shell);	
		
		robot.focus(tf2);
		Robot.waitForIdle(shell.getDisplay());	
		assertEquals("Wrong focus detected",robot.findFocusOwner(shell.getDisplay()),tf2);
									
		hideShell(shell);
	}

	public void testIconify(){
		Shell shell = createShell();
		showShell(shell);
		
		robot.iconify(shell);
		Timer timer = new Timer();
		while (!shell.getMinimized()) {
			if (timer.elapsed() > 2000)
				fail("Shell not iconified");
			Robot.sleep();
		}
		robot.deiconify(shell);
		timer.reset();
		while (shell.getMinimized()) {
			if (timer.elapsed() > 2000)
				fail("Shell not restored");
			Robot.sleep();
		}
		hideShell(shell);	
	}
	
	public void testMaximize(){
		Shell shell = createShell();
		showShell(shell);
		Rectangle bounds = WidgetLocator.getBounds(shell,false);
		
		robot.maximize(shell);
		Timer timer = new Timer();
		while (bounds.equals(WidgetLocator.getBounds(shell,false))) {
		   if (timer.elapsed() > 2000)
			   fail("Shell not maximized");
			Robot.sleep();
	    }
	   	
	   	bounds = WidgetLocator.getBounds(shell,false);
	   	robot.normalize(shell);
		timer.reset();
		while (bounds.equals(WidgetLocator.getBounds(shell,false))) {
			if (timer.elapsed() > 2000)
				fail("Shell not restored");
			Robot.sleep();
		}
		
		hideShell(shell);
	}


	public void testMethodsFromSeparateThread(){
		Thread.currentThread().setName("Display Thread");
		final Shell shell =createShell();
		final Menu bar = new Menu(shell,SWT.BAR);
		shell.setMenuBar(bar);
		MenuItem file = new MenuItem(bar,SWT.CASCADE);
		file.setText("File");
		Menu menu = new Menu(shell,SWT.DROP_DOWN);
		file.setMenu(menu);
		MenuItem one = new MenuItem(menu,SWT.CHECK);
		one.setText("One");
		MenuItem two = new MenuItem(menu,SWT.CHECK);
		two.setText("Two");
		two.setData("name","Two");
		MenuItem  subMenuItem = new MenuItem(menu,SWT.CASCADE);
		subMenuItem.setText("Sub-Menu");
		Menu subMenu = new Menu(shell,SWT.DROP_DOWN);
		subMenuItem.setMenu(subMenu);	
		MenuItem sub1 = new MenuItem(subMenu,SWT.CHECK);
		sub1.setText("Sub1");
		final MenuItem sub2 = new MenuItem(subMenu,SWT.CHECK);
		sub2.setText("Sub2");
		sub2.setData("name","Sub2");
						
		final Decorations dec = new Decorations(shell,SWT.NONE);
		dec.setSize(50,50);
		//label.setSize(50,50);
		dec.setBackground(new Color(dec.getDisplay(),255,255,255));
		
		Menu popup = new Menu(dec,SWT.POP_UP);
		dec.setMenu(popup);
		MenuItem popItem = new MenuItem(popup,SWT.CHECK);
		popItem.setText("pop item");
				
		showShell(shell);
		Robot.delay(1000);
		
		Runnable r;
		
		r = new Runnable(){
			public void run(){
				// put method here
				robot.mouseMove(dec);
			}
		};
		assertTrue("mouseMove(Widget) failed to return",runInNewThread(r, shell.getDisplay()));
		
		r = new Runnable(){
			public void run(){
				// put method here
				Robot.waitForIdle(shell.getDisplay());
			}
		};
		assertTrue("waitForIdle() failed to return",runInNewThread(r, shell.getDisplay()));

		r = new Runnable(){
			public void run(){
				// put method here
				robot.mouseMove(dec,20,20);
			}
		};
		assertTrue("mouseMove(Widget,int,int) failed to return",runInNewThread(r, shell.getDisplay()));
		
		r = new Runnable(){
			public void run(){
				// put method here
				robot.findFocusOwner(shell.getDisplay());
			}
		};
		assertTrue("findFocusOwner() failed to return",runInNewThread(r, shell.getDisplay()));
		
		r = new Runnable(){
			public void run(){
				// put method here
				robot.focus(dec);
			}
		};
		assertTrue("focus(Control) failed to return",runInNewThread(r, shell.getDisplay()));
		
		r = new Runnable(){
			public void run(){
				// put method here
				robot.key('a');
			}
		};
		assertTrue("key(int) failed to return",runInNewThread(r, shell.getDisplay()));								
			
		r = new Runnable(){
			public void run(){
				// put method here
				robot.capture(dec);
			}
		};
		assertTrue("capture(Widget) failed to return",runInNewThread(r, shell.getDisplay()));		
		
		r = new Runnable(){
			public void run(){
				// put method here
				robot.click(dec);
			}
		};
		assertTrue("click(Widget) failed to return",runInNewThread(r, shell.getDisplay()));			

		r = new Runnable(){
			public void run(){
				// put method here
				robot.selectMenuItem(sub2);
			}
		};
		assertTrue("selectMenuItem test failed to return",runInNewThread(r, shell.getDisplay()));	
		
		r = new Runnable(){
			public void run(){
				// put method here
				robot.selectMenuItemByText(bar,"Two");
			}
		};
		assertTrue("selectMenuItemByText test failed to return",runInNewThread(r, shell.getDisplay()));			

		//r = new Runnable(){
		//	public void run(){
		//		// put method here
		//		robot.showPopupMenu(dec);
		//	}
		//};
		//assertTrue("showPopupMenu(Control) test failed to return",runInNewThread(r));			
			
		hideShell(shell);	
	}
	
	/**
	 * returns true iff method executed
	 */
	public boolean runInNewThread(final Runnable action, Display display){
		flag = false;
		Thread t = new Thread(){
			public void run(){
				action.run();
				flag = true;
			}
		};
		t.setName("Testing thread");
		t.start();
		
		Timer timer = new Timer();
		while(!flag && timer.elapsed()<5000){
			display.readAndDispatch();
		}
		
		return flag;
	}
				
	/* Due to what I believe is a Windows 2000/XP peculiarity, Display.readAndDispatch()
	 * will not return until all menus are closed.  So it is impossible to reach a
	 * point in the display's code where a menu has been programmatically left open	 * 
	 */
	//public void testMenuSelectionWithParentShowing(){}
		

	public static void main(String[] argv){
		RepeatHelper.runTests(argv, RobotTest.class);
	}
}
