
package com.ibm.abbot.tester.swt;


import junit.extensions.abbot.*;

import org.eclipse.swt.graphics.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.*;

/** Unit test to verify the base WidgetTester class. */

// OSX uses a button/modifier combo to indicate MB2/3...ick
// w32 uses a button/modifier combo to indicate MB3...ick
public class WidgetTesterTest extends WidgetTestFixture{

	private WidgetTester tester;

	protected void setUp() {
		tester = new WidgetTester();
	}
	protected void tearDown() {
		tester = null;
	}
	
	/** Create a new test case with the given name. */
	public WidgetTesterTest(String name) {
		super(name);
	}

	public void testSelectMenuItem() {

		Shell shell = createShell();
		Menu bar = new Menu(shell, SWT.BAR);
		shell.setMenuBar(bar);
		MenuItem file = new MenuItem(bar, SWT.CASCADE);
		file.setText("File");
		Menu menu = new Menu(shell, SWT.DROP_DOWN);
		file.setMenu(menu);
		MenuItem one = new MenuItem(menu, SWT.CHECK);
		one.setText("One");
		MenuItem two = new MenuItem(menu, SWT.CHECK);
		two.setText("Two");
		two.setData("name", "Two");
		MenuItem subMenuItem = new MenuItem(menu, SWT.CASCADE);
		subMenuItem.setText("Sub-Menu");
		Menu subMenu = new Menu(shell, SWT.DROP_DOWN);
		subMenuItem.setMenu(subMenu);
		MenuItem sub1 = new MenuItem(subMenu, SWT.CHECK);
		sub1.setText("Sub1");
		final MenuItem sub2 = new MenuItem(subMenu, SWT.CHECK);
		sub2.setText("Sub2");
		sub2.setData("name", "Sub2");

		Menu popup = new Menu(shell, SWT.POP_UP);
		shell.setMenu(popup);
		MenuItem pop1 = new MenuItem(popup, SWT.CHECK);
		pop1.setText("Pop1");
		pop1.setData("name", "Pop1");
		MenuItem pop2 = new MenuItem(popup, SWT.CHECK);
		pop2.setText("Pop2");
		pop2.setData("name", "Pop2");

		showShell(shell);

		one.setSelection(false);
		tester.actionSelectMenuItemByText(bar, "One");
		assertTrue("First menu item not selected", one.getSelection());
		sub2.setSelection(false);
		tester.actionSelectMenuItemByText(bar, "Sub2");
		assertTrue("Second menu item not selected", sub2.getSelection());
		pop1.setSelection(false);
		tester.actionSelectMenuItemByText(popup, "Pop1");
		assertTrue(
			"First popup submenu item not selected",
			pop1.getSelection());
		pop2.setSelection(false);
		tester.actionSelectMenuItemByText(popup, "Pop2");
		assertTrue(
			"Second popup submenu item not selected",
			pop2.getSelection());

		hideShell(shell);
	}

	boolean started = false, finished = false;
	String title0, shell0Title;
	public void testDecorationsShowingAssertion() {
		setDelay(1000);
		Shell shell = createShell();
		Decorations dec = new Decorations(shell, SWT.TITLE);
		dec.setText("Dec");

		showShell(shell);

		String title = shell.getText();
		// shell should be showing at this point
		assertTrue(
			"Shell'" + title + "' wasn't showing or wasn't detected",
			tester.assertDecorationsShowing(title));
		// dec should be showing too
		assertTrue(
			"Dec'" + dec.getText() + "' wasn't showing or wasn't detected",
			tester.assertDecorationsShowing(dec.getText()));

		// now test a Decorations that is a child of a shell from a different thread
		Thread newThread = new Thread() {
			public void run() {
				Display display0 = new Display();
				Shell shell0 = new Shell(display0, SWT.SHELL_TRIM);
				Decorations dec0 = new Decorations(shell0, SWT.TITLE);
				dec0.setText("Dec0");
				shell0.setText("testDecorationsShowingAssertion0");

				shell0.setSize(DEFAULT_SHELL_WIDTH, DEFAULT_SHELL_HEIGHT);
				showShell(shell0);
				tester.actionFocus(dec0);
				shell0Title = shell0.getText();
				shell0.setLocation(50, 50);
				title0 = dec0.getText();
				started = true;

				while (!finished) {
					display0.readAndDispatch();
				}
				hideShell(shell0);
				display0.dispose();
				while (!display0.isDisposed());
			}
		};

		newThread.start();
		while (!started)
			shell.getDisplay().readAndDispatch();
		// dec0 should now be showing
		assertTrue(
			"Decorations'" + title0 + "' wasn't showing or wasn't detected",
			tester.assertDecorationsShowing(title0));
		// shell0 should now be showing
		assertTrue(
			"Shell'" + shell0Title + "' wasn't showing or wasn't detected",
			tester.assertDecorationsShowing(shell0Title));
		finished = true;
		try {
			newThread.join();
		} catch (Exception e) {
		}
		hideShell(shell);
	}

	public void testDecorationsShowing() {
		Shell shell = createShell();

		showShell(shell);

		String title = shell.getText();

		assertEquals(
			"Shell not showing",
			true,
			tester.assertDecorationsShowing(title));
		shell.dispose();
		assertEquals(
			"Shell still showing",
			false,
			tester.assertDecorationsShowing(title));
		hideShell(shell);
	}

	public void testKeyStroke() {
		
		Shell shell = createShell();
		Text text = new Text(shell, SWT.NONE);
		text.setSize(150, 20);
		text.setLocation(0, 0);
		KeyWatcher kw = new KeyWatcher();
		text.addKeyListener(kw);

		showShell(shell);
		
		shell.setMinimized(true);	// HACK- I dunno why this is nec, but
		shell.setMinimized(false);	// the shell, and all others created after it
									// will start off, and stay, minimized w/out this
		int mods = SWT.SHIFT;
		tester.actionFocus(text);
		tester.actionKeyChar('A',text.getDisplay());
		tester.actionWaitForIdle(shell.getDisplay());
		assertTrue("Never received key press", kw.gotPress);
		assertTrue("Never received release", kw.gotRelease);
		assertTrue("Never received type", kw.gotTyped);
		assertEquals("Wrong modifiers", mods, kw.modifiers);

		hideShell(shell);
	}

	public void testDoubleClick() {
		MouseWatcher mw = new MouseWatcher();
		Shell shell = createShell();
		Text text = new Text(shell, SWT.NONE);
		text.setSize(150, 20);
		text.setLocation(0, 0);
		text.addMouseListener(mw);

		showShell(shell);

		//tester.actionFocus(text);
		Rectangle bounds = WidgetLocator.getBounds(text);
		tester.actionClick(
			text,
			bounds.width / 2,
			bounds.height / 2,
			"BUTTON1",
			2);

		hideShell(shell);
	}

	/** Ensure that we can send clicks close enough together to register as a
		double click. */
	public void testTwoClicks() {
		MouseWatcher mw = new MouseWatcher();
		Shell shell = createShell();
		Text text = new Text(shell, SWT.NONE);
		text.setSize(150, 20);
		text.setLocation(0, 0);
		text.addMouseListener(mw);

		showShell(shell);

		tester.actionFocus(text);
		tester.actionClick(text);
		tester.actionClick(text);
		assertEquals("Wrong number of clicks", 2, mw.clickCount);

		hideShell(shell);
	}

	public void testDeriveTag() {
		Shell shell = createShell();
		Button button = new Button(shell, SWT.PUSH);
		button.setText("BUTTON");

		showShell(shell);

		assertEquals(
			"Tag should be the Button's label",
			button.getText(),
			tester.deriveTag(button));
		assertEquals(
			"Tag should be the JFrame's title",
			shell.getText(),
			tester.deriveTag(shell));
		hideShell(shell);
	}

	public void testStripHTML() {
		String html1 = "<html>simple</html>";
		String strip1 = "simple";
		String html2 =
			"<html><font color=\"red\" size=\"-1\">complex</font></html>";
		String strip2 = "complex";
		assertEquals(
			"Simple html was not stripped",
			strip1,
			WidgetTester.stripHTML(html1));
		assertEquals(
			"Complex html was not stripped",
			strip2,
			WidgetTester.stripHTML(html2));
	}

	private boolean gotDrag = false;
	public void testDragDrop() {
		DragWatcher dw = new DragWatcher();
		Shell shell = createShell();
		Label label = new Label(shell, SWT.BORDER);
		label.setText("LABEL");
		label.setLocation(0, 0);
		label.addListener(SWT.DragDetect, dw);

		showShell(shell);

		Rectangle bounds = WidgetLocator.getBounds(label);
		tester.actionDrag(label, 1, 1);
		tester.actionDrop(label, bounds.width / 2, bounds.height / 2);
		assertTrue("Should have seen a drag event", dw.gotDragStart);

		hideShell(shell);
	}
	
	public void testAssertImage(){
		Shell shell = createShell();
		Label label = new Label(shell,SWT.CENTER);
		Image image = new Image(shell.getDisplay(),RobotTest.TEST_IMAGE_DIR+"\\image.jpg");
		label.setImage(image);
		
		showShell(shell);
		shell.pack();
		tester.actionWaitForIdle(shell.getDisplay());
		tester.actionDelay(500);		
	
		assertTrue("Failed to assert that images match", tester.assertImage(label,image,true));
		hideShell(shell);
	}

/** I Think that this test must be run outside of Eclipse, b/c I think
 * 	eclipse automatically loads all classes in a project... so we can't
 *  use abbot's custom classLoader
 */
	/** Assumes "classes" has the class in question. */
//	public void testGetTester() throws Throwable {
//		System.out.println(WidgetClassForLoading.class.getClassLoader());
//		String cname = "test.fake.WidgetClassForLoading";
//		ClassLoader cl = 
//			new abbot.util.PreloadingClassLoader("classes", getClass().getClassLoader());
//		System.out.println(cl);
//
//		Class cls = Class.forName(cname, true, cl);
//		System.out.println(cls.getClassLoader());
//		assertEquals("Wrong class loader for test class",
//					 cl, cls.getClassLoader());
//		WidgetTester customTester = WidgetTester.getTester(cls);
//		WidgetTester tester =
//			WidgetTester.getTester(org.eclipse.swt.widgets.Widget.class);
//		assertEquals("Wrong tester loaded", tester, customTester);
//		assertTrue("Class loaders should be different",
//				   !cls.getClassLoader().equals(tester.getClass().getClassLoader()));
//	}

		
	private class MouseWatcher extends MouseAdapter{
		public boolean gotPress = false;
		public boolean gotRelease = false;
		public boolean gotClick = false;
		public boolean gotDoubleClick = false;
		public boolean popupTrigger = false;
		public int clickCount = 0;
		public Widget source = null;
		public int modifiers = 0;
		public Point where = null;
	
		public void mouseDown(MouseEvent me){
			where = new Point(me.x,me.y);
			source = me.widget;
			gotPress = true;
			modifiers = me.stateMask;			
		}
		public void mouseUp(MouseEvent me){
			where = new Point(me.x,me.y);
			source = me.widget;
			modifiers = me.stateMask;
			gotRelease = true;
			if(gotPress)
				mouseClick(me);
		}
		public void mouseClick(MouseEvent me){
			where = new Point(me.x,me.y);
			if(Robot.POPUP_MASK==SWT.BUTTON2 && me.button==2 ||
			   Robot.POPUP_MASK==SWT.BUTTON3 && me.button==3)
				popupTrigger = true;
			gotClick = true;
			clickCount++;
			modifiers = me.stateMask;			
		}
		public void mouseDoubleClick(MouseEvent me){
			source = me.widget;
			modifiers = me.stateMask;			
			where = new Point(me.x,me.y);
			gotDoubleClick = true;
			if(Robot.POPUP_MASK==SWT.BUTTON2 && me.button==2 ||
			   Robot.POPUP_MASK==SWT.BUTTON3 && me.button==3)
				popupTrigger = true;
		}
	};


	private class KeyWatcher extends KeyAdapter {
	   public boolean gotPress = false;
	   public boolean gotRelease = false;
	   public boolean gotTyped = false;
	   public int keyCode = 0;
	   public char character;
	   public int modifiers = 0;
	   public void keyPressed(KeyEvent ke) {
		   gotPress = true;
		   keyCode = ke.keyCode;
		   modifiers = ke.stateMask;
		   character = ke.character;
	   }
	   public void keyReleased(KeyEvent ke) {
		   gotRelease = true;
		   if(gotPress)
			 gotTyped = true;
		   character = ke.character;
	   }
	   public void keyTyped(KeyEvent ke) {
		   gotTyped = true;
		   character = ke.character;
	   }
	};

	private class DragWatcher implements Listener {
		public boolean 	gotDragStart = false,
						gotSetData = false,
						gotDragFinish = false;
		public void handleEvent(Event e){
			gotDragStart = true;
		}
	};

	public static void main(String[] args) {
		RepeatHelper.runTests(args, WidgetTesterTest.class);
	}	
	
}


